=begin

(c) RENDERIZA 2013

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.

== Information
Author:: Renderiza
Name:: PanoScene
Version:: 1.0.0
SU Version:: v8
Date:: 7/20/2013
Description:: Create Panoramic Scenes.
History::
1.0.0:: 7/20/2013
	* Original release.  
=end

module RND_Extensions

	module RND_Panoscene

		class Panoscene

			@@model = Sketchup.active_model
			@@path = File.dirname(__FILE__) # this plugin's folder
			@@rnd_panoscene = "rnd_panoscene.html"
			@@panoscene_files = File.join(@@path, '/PanosceneFiles/')
			@@pano = File.join(@@panoscene_files, 'PanoScene.skp')
			@@panoscene_dlg = nil

			def initialize()

				@pano_1_file =  File.join(@@path, @@rnd_panoscene)

				if (!@pano_1_file) then
					UI.messagebox("Cannot find file '#{@@rnd_panoscene} in folder '#{@@path}'")
					return
				end

				if @@panoscene_dlg == nil
					@@panoscene_dlg = UI::WebDialog.new("PanoScene", false, "PanoScene", 195, 800, 70, 95, true)
					@@panoscene_dlg.add_action_callback("push_frame") do |d,p|
					push_frame(d,p)
					end
				end

				
				@@panoscene_dlg.set_size(195, 800)
				@@panoscene_dlg.set_file(@pano_1_file)
				@@panoscene_dlg.show()
				
				model = Sketchup.active_model
				ents = model.active_entities
				groups = ents.grep(Sketchup::ComponentInstance)

					groups.each do |e|
						if e.definition.name == "Camera"
							@create = false
							break
						else
							@create = true 
						end
					end
				
				if @create == true
					point = Geom::Point3d.new 0,0,0
					transform = Geom::Transformation.new point
					model = Sketchup.active_model
					entities = model.active_entities
					
					definitions = model.definitions
					componentdefinition = definitions.load @@pano
					name = componentdefinition.name="Camera"
					instance = entities.add_instance componentdefinition, transform
					UI.messagebox("Position 'PanoScene' object where you want the camera position.")
					Sketchup.send_action('selectSelectionTool:') #Selection Tool
				end
				
			end #def
			


			def push_frame(dialog,data)

				params = query_to_hash(data) # parse state information

				########################################
				###Click Create#########################
				########################################
				if params['active'].to_s == "1"

					Sketchup.active_model.select_tool RND_Extensions::RND_Panoscene::Panoscene.new
				
					#########
					###RUN###
					#########
				
					model = Sketchup.active_model
					view = model.active_view
					camera = view.camera
					eye = camera.eye

					pages = model.pages
					page = pages.selected_page
					sel = model.selection
					ents = model.active_entities
					groups = ents.grep(Sketchup::ComponentInstance)

					groups.each do |e|
						if e.definition.name == "Camera"
							e.name = "Camera"
							boundbox = e.bounds
							@center = boundbox.center
						end
					end
					
					height = params['getheight'].to_f
					x = @center.x.to_f
					y = @center.y.to_f
					z = height*12
					newEye = Geom::Point3d.new(x,y,z)
					newUp = Geom::Vector3d.new(0,0,1)
					camdist = height*12

					if page == nil
						pages.add "Default"
					end

					left180  = "a_-180°"
					left157  = "b_-157.5°"
					left135  = "c_-135°"
					left112  = "d_-112.5°"
					left90   = "e_-90°"
					left67   = "f_-67.5°"
					left45   = "g_-45°"
					left22   = "h_-22.5°"
					center0  = "i_0°"
					right22  = "j_22.5°"
					right45  = "k_45°"
					right67  = "l_67.5°"
					right90  = "m_90°"
					right112 = "n_112.5°"
					right135 = "o_135°"
					right157 = "p_157.5°"
					back180  = "q_180°"
					page_num = 0

					#####################################################################
					# -180° #############################################################
					#####################################################################
					if params['a'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left180
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "a_-180°"
						pages.add "a_-180°"
						end

						targetX = x
						targetY = y-camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# -157.5° ###########################################################
					#####################################################################
					if params['b'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left157
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "b_-157.5°"
						pages.add "b_-157.5°"
						end

						targetX = x-camdist/2
						targetY = y-camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# -135° #############################################################
					#####################################################################
					if params['c'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left135
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "c_-135°"
						pages.add "c_-135°"
						end

						targetX = x-camdist
						targetY = y-camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# -112.5° ###########################################################
					#####################################################################
					if params['d'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left112
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "d_-112.5°"
						pages.add "d_-112.5°"
						end

						targetX = x-camdist
						targetY = y-camdist/2
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# -90° ##############################################################
					#####################################################################
					if params['ee'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left90
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "e_-90°"
						pages.add "e_-90°"
						end

						targetX = x-camdist
						targetY = y
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# -67.5° ############################################################
					#####################################################################
					if params['f'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left67
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "f_-67.5°"
						pages.add "f_-67.5°"
						end

						targetX = x-camdist
						targetY = y+camdist/2
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# -45° ##############################################################
					#####################################################################
					if params['g'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left45
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "g_-45°"
						pages.add "g_-45°"
						end

						targetX = x-camdist
						targetY = y+camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# -22.5° ############################################################
					#####################################################################
					if params['h'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == left22
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "h_-22.5°"
						pages.add "h_-22.5°"
						end

						targetX = x-camdist/2
						targetY = y+camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					#  0° ###############################################################
					#####################################################################
					if params['i'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == center0
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "i_0°"
						pages.add "i_0°"
						end

						targetX = x
						targetY = y+camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevACam = view.camera
						elevACam.set(newEye,newTarget,newUp)

						view.camera = elevACam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# 22.5° #############################################################
					#####################################################################
					if params['j'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == right22
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "j_22.5°"
						pages.add "j_22.5°"
						end

						targetX = x+camdist/2
						targetY = y+camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# 45° ###############################################################
					#####################################################################
					if params['k'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == right45
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "k_45°"
						pages.add "k_45°"
						end

						targetX = x+camdist
						targetY = y+camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					# 67.5° #############################################################
					#####################################################################
					if params['l'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == right67
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "l_67.5°"
						pages.add "l_67.5°"
						end

						targetX = x+camdist
						targetY = y+camdist/2
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					#  90° ##############################################################
					#####################################################################
					if params['m'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == right90
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "m_90°"
						pages.add "m_90°"
						end

						targetX = x+camdist
						targetY = y
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					#  112° #############################################################
					#####################################################################
					if params['n'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == right112
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "n_112.5°"
						pages.add "n_112.5°"
						end

						targetX = x+camdist
						targetY = y-camdist/2
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					#  135° #############################################################
					#####################################################################
					if params['o'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == right135
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "o_135°"
						pages.add "o_135°"
						end

						targetX = x+camdist
						targetY = y-camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					#  157° #############################################################
					#####################################################################
					if params['p'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == right157
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "p_157.5°"
						pages.add "o_157.5°"
						end

						targetX = x+camdist/2
						targetY = y-camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					#  180° #############################################################
					#####################################################################
					if params['q'].to_s == "true"
						x = @center.x.to_f
						y = @center.y.to_f
						page_num = 0
						pages.each {|page|
							if page.name == back180
								pages.selected_page = model.pages[page_num]
								break
							end
							page_num+=1
						} #Pages.each
						if page.name != "q_180°"
						pages.add "q_180°"
						end

						targetX = x
						targetY = y-camdist
						targetZ = z
						newTarget = Geom::Point3d.new(targetX,targetY,targetZ)

						elevBCam = view.camera
						elevBCam.set(newEye,newTarget,newUp)

						view.camera = elevBCam
						view.invalidate()

						camera.perspective = true
						page = pages.selected_page.update(1)
					end
					#####################################################################
					page_num = 0
					pages.each {|page|
						if page.name == center0
							pages.selected_page = model.pages[page_num]
							break
						end
						page_num+=1
					} #Pages.each
					#####################################################################
					
					
					activate = 0
					script = "top.active = " + activate.to_s + ";"
					dialog.execute_script(script)

					Sketchup.send_action('selectSelectionTool:') #Selection Tool

				end #Done

				########################################
				###Click Done###########################
				########################################
				if params['done'].to_s == "1"
					Sketchup.send_action('selectSelectionTool:') #Selection Tool
					@@panoscene_dlg.close
				end

			end #def



			def unescape(string)
				if string != nil
					string = string.gsub(/\+/, ' ').gsub(/((?:%[0-9a-fA-F]{2})+)/n) do
						[$1.delete('%')].pack('H*')
					end
				end
				return string
			end #def



			def query_to_hash(query)
				param_pairs = query.split('&')
				param_hash = {}
				for param in param_pairs
					name, value = param.split('=')
					name = unescape(name)
					value = unescape(value)
					param_hash[name] = value
				end
				return param_hash
			end #def

		end #class

	end #module

end #module

file_loaded( __FILE__ )